##################################################################################
#
#   Kaivo Public Software License
#
#   Copyright (c) 2001, Kaivo, Inc.
#   All rights reserved.
#
#   Redistribution and use in source and binary forms, with or without
#   modification, are permitted provided that the following conditions are met:
#
#       o Redistributions of source code must retain the above copyright notice,
#         this list of conditions and the following disclaimer.
#
#       o Redistributions in binary form must reproduce the above copyright
#         notice, this list of conditions and the following disclaimer
#         in the documentation and/or other materials provided with the
#         distribution.
#
#       o Neither Kaivo nor the names of its contributors may be used to endorse
#         or promote products derived from this software without specific prior
#         written permission.
#
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#   ARE DISCLAIMED. IN NO EVENT SHALL KAIVO OR ITS CONTRIBUTORS BE LIABLE FOR 
#   ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
#   (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
#   LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND 
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
#   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
#   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##################################################################################

# Eval tag by Casey Duncan (cduncan@kaivo.com)
# Kaivo, Inc. (http://www.kaivo.com)

__doc__ = 'DTML eval multiple expression tag'
__version__ = '0.2'

from DocumentTemplate.DT_Util import Eval, html_quote
from DocumentTemplate.DT_String import String
from string import split, strip
import re

class EvalTag:
    """dtml-eval tag"""

    name = 'eval'
    blockContinuations = ()
    disable_dtml_block_parsing = 1
    assign_re = re.compile('\s*([A-Za-z][A-Za-z0-9_]*)\s*=([^=].*)')
    # assign_re = regex.compile('[ \t]*\([A-Za-z][A-Za-z0-9_]*\)[\t]*=\([^=].*\)')


    def __init__(self, blocks):
        tag, args, section = blocks[0]
        self.section = section

    def render(self, md):
        eval_list = split(self.section.read(),'\n')
        try: REQUEST = md['REQUEST']
        except KeyError: REQUEST = noREQUEST()            
        prev_line_str = ''

        for eval_line in eval_list:
            if not prev_line_str:
                # Line not continued from last
                assign_match = self.assign_re.match(eval_line)
                if assign_match:
                    # Variable assignment
                    assign_name = assign_match.group(1)
                    expr_str = assign_match.group(2)
                else:
                    # Simple Expression
                    assign_name = ""
                    expr_str = strip(eval_line)
            else:
                # Line is continued from last
                expr_str = prev_line_str + strip(eval_line)
                prev_line_str = ""
 
            if expr_str:
                if expr_str[-1] == "\\":
                    # This line continues to the next one
                    prev_line_str = expr_str[:-1]
                    continue

                try: 
                    expr = Eval(expr_str)
                except SyntaxError, err:
                    raise "SyntaxError", """\
                        <strong>Expression (Python) Syntax error: %s</strong>:
                        \n<pre>\n%s\n</pre>\n
                        <strong>In Eval Tag</strong>""" % (err, html_quote(expr_str))
                
                if assign_name: REQUEST.set(assign_name, expr.eval(md))
                else: expr.eval(md)

        return ''

        if prev_line_str:
            raise "SyntaxError", """\
                <strong>Expression (Python) Syntax error: Line continued past end</strong>:
                \n<pre>\n%s\n</pre>\n
                <strong>In Eval Tag</strong>""" % html_quote(eval_line)

    __call__ = render


# Install the eval tag globally

String.commands['eval'] = EvalTag

class noREQUEST:
    def set(self, x, y):
        raise "AssignmentError", """\
            Cannot set a variable in eval tag without the REQUEST object."""
